@@ -26,7 +26,7 @@ from torch_geometric.data.data import Data
 
 from .gnn_training_utils import check_if_graph_is_connected, pass_data_iteratively
 from .dataset import generate, load_OMICS_dataset, convert_to_s2vgraph
-from .gnn_expaliner import GNNExplainer
+from .gnn_explainer import GNNExplainer
 from .graphcnn import GraphCNN
 
 from .community_detection import find_communities
@@ -37,7 +37,7 @@ class GNNSubNet(object):
     The class GNNSubSet represents the main user API for the
     GNN-SubNet package.
     """
-    def __init__(self, location, ppi=None, features=None, target=None, cutoff=950, normalize=True) -> None:
+    def __init__(self, location=None, ppi=None, features=None, target=None, cutoff=950, normalize=True) -> None:
 
         self.location = location
         self.ppi = ppi
@@ -54,6 +54,9 @@ class GNNSubNet(object):
         # Flags for internal use (hidden from user)
         self._explainer_run = False
 
+        if ppi == None:
+            return None
+
         dataset, gene_names = load_OMICS_dataset(self.ppi, self.features, self.target, True, cutoff, normalize)
 
          # Check whether graph is connected
@@ -63,7 +66,7 @@ class GNNSubNet(object):
         if check == False:
 
             print("Calculate subgraph ...")
-            dataset, gene_names = load_OMICS_dataset(self.ppi, self.features, self.target, True, cutoff, normalize)
+            dataset, gene_names = load_OMICS_dataset(self.ppi, self.features, self.target, False, cutoff, normalize)
 
         check = check_if_graph_is_connected(dataset[0].edge_index)
         print("Graph is connected ", check)
@@ -97,14 +100,15 @@ class GNNSubNet(object):
         #for i in self.__dict__:
         #    print('%s: %s' % (i, self.__dict__[i]))
 
-    def train(self, epoch_nr = 10, shuffle=True, weights=None):
+    def train(self, epoch_nr = 10, shuffle=True, weights=False):
         """
         Train the GNN model on the data provided during initialisation.
         """
+        use_weights = False
+
         dataset = self.dataset
         gene_names = self.gene_names
 
-
         graphs_class_0_list = []
         graphs_class_1_list = []
         for graph in dataset:
@@ -130,11 +134,9 @@ class GNNSubNet(object):
             random_graphs_class_1_list = random.sample(graphs_class_1_list, graphs_class_0_len)
             balanced_dataset_list = graphs_class_0_list + random_graphs_class_1_list
 
-
         #print(len(random_graphs_class_0_list))
         #print(len(random_graphs_class_1_list))
 
-
         random.shuffle(balanced_dataset_list)
         print(f"Length of balanced dataset list: {len(balanced_dataset_list)}")
 
@@ -171,8 +173,6 @@ class GNNSubNet(object):
         #for item in dataset:
         #    count += item.y.item()
 
-        use_weights = False
-
         #weight = torch.tensor([count/len(dataset), 1-count/len(dataset)])
         #print(count/len(dataset), 1-count/len(dataset))
 
@@ -180,8 +180,6 @@ class GNNSubNet(object):
         no_of_features = dataset[0].x.shape[1]
         nodes_per_graph_nr = dataset[0].x.shape[0]
 
-        load_model = False
-
         #print(len(dataset), len(dataset)*0.2)
         #s2v_dataset = convert_to_s2vgraph(dataset)
         #train_dataset, test_dataset = train_test_split(dataset, test_size=0.2, random_state=123)
@@ -189,13 +187,13 @@ class GNNSubNet(object):
         #s2v_test_dataset = convert_to_s2vgraph(test_dataset)
         #s2v_train_dataset, s2v_test_dataset = train_test_split(s2v_dataset, test_size=0.2, random_state=123)
 
-
         input_dim = no_of_features
         n_classes = 2
 
         model = GraphCNN(5, 2, input_dim, 32, n_classes, 0.5, True, 'sum1', 'sum', 0)
         opt = torch.optim.Adam(model.parameters(), lr = 0.01)
 
+        load_model = False
         if load_model:
             checkpoint = torch.load(model_path)
             model.load_state_dict(checkpoint['state_dict'])
@@ -318,11 +316,13 @@ class GNNSubNet(object):
         model.train()
 
         self.model_status = 'Trained'
-        self.model = model
+        self.model = copy.deepcopy(model)
         self.accuracy = accuracy
         self.confusion_matrix = confusion_matrix_gnn
         self.test_loss = test_loss
         self.s2v_test_dataset = s2v_test_dataset
+        self.predictions = predicted_class_array
+        self.true_class  = true_class_array
 
     def explain(self, n_runs=10, explainer_lambda=0.8, save_to_disk=False):
         """
@@ -371,6 +371,8 @@ class GNNSubNet(object):
         self.node_mask_matrix = np.concatenate(NODE_MASK,1)
         self.node_mask = np.concatenate(NODE_MASK,1).mean(1)
 
+        self._explainer_run = True
+        
         ###############################################
         # Perform Community Detection
         ###############################################
@@ -403,6 +405,48 @@ class GNNSubNet(object):
 
         self._explainer_run = True
 
+    def predict(self, gnnsubnet_test):
+
+        confusion_array = []
+        true_class_array = []
+        predicted_class_array = []
+
+        s2v_test_dataset  = convert_to_s2vgraph(gnnsubnet_test.dataset)
+        model = self.model
+        model.eval()
+        output = pass_data_iteratively(model, s2v_test_dataset)
+        predicted_class = output.max(1, keepdim=True)[1]
+        labels = torch.LongTensor([graph.label for graph in s2v_test_dataset])
+        correct = predicted_class.eq(labels.view_as(predicted_class)).sum().item()
+        acc_test = correct / float(len(s2v_test_dataset))
+
+        #if use_weights:
+        #    loss = nn.CrossEntropyLoss(weight=weight)(output,labels)
+        #else:
+        #    loss = nn.CrossEntropyLoss()(output,labels)
+        #test_loss = loss
+
+        predicted_class_array = np.append(predicted_class_array, predicted_class)
+        true_class_array = np.append(true_class_array, labels)
+
+        confusion_matrix_gnn = confusion_matrix(true_class_array, predicted_class_array)
+        print("\nConfusion matrix:\n")
+        print(confusion_matrix_gnn)
+
+        counter = 0
+        for it, i in zip(predicted_class_array, range(len(predicted_class_array))):
+            if it == true_class_array[i]:
+                counter += 1
+
+        accuracy = counter/len(true_class_array) * 100
+        print("Accuracy: {}%".format(accuracy))
+        
+        self.predictions_test = predicted_class_array
+        self.true_class_test  = true_class_array
+        self.accuracy_test = accuracy
+        self.confusion_matrix_test = confusion_matrix_gnn
+
+
     def download_TCGA(self, save_to_disk=False) -> None:
         """
         Warning: Currently not implemented!
